@echo off
chcp 1252>nul

rem **************************************************
rem Angelo.bat
SET Version=1.5
rem Date 2023-09-19
rem by Nils Kaczenski, faq-o-matic.net
rem This script creates automated health reports
rem of an Active Directory environment
rem It uses a number of external tools
rem Warning: This script might create significant
rem load in large AD environments
rem **************************************************

rem Change to script directory
cd %0\..

rem Define environment variables
set ADir=%CD%
set DC=%logonserver:~2%
set DCDNS=%DC%.%userdnsdomain%
set DOM=%userdomain%
set DOMDNS=%userdnsdomain%
for /f "Tokens=2 Delims=[] skip=1" %%i in ('ping -n 1 -4 %DCDNS%') do set DCIP=%%i
REM Create time stamp
SET DateTempFile=%RANDOM%Temp.vbs
CD %0\..
ECHO dtmNow = Now()>%DateTempFile%
ECHO strDate = Year(dtmNow) ^& Right("0" ^& Month(dtmNow),2) ^& Right("0" ^& Day(dtmNow), 2) ^& "-" ^& Right("0" ^& Hour(dtmNow), 2) ^& Right("0" ^& Minute(dtmNow), 2) ^& Right("0" ^& Second(dtmNow), 2)>>%DateTempFile%
ECHO WScript.Echo strDate>>%DateTempFile%
FOR /F " usebackq delims==" %%a IN (`@CSCRIPT //NOLOGO %DateTempFile%`) DO (
	SET MyDate=%%a
)
DEL %DateTempFile%
SET RepDir=Reports\Report-%DOM%-%MyDate%

echo Angelo: AD Diagnosis and Documentation
echo ======================================
echo.
echo Angelo version: %Version%
echo Start time: %time%
echo Working directory: %ADir%
echo Domain: %DOM%
echo Domain DNS name: %DOMDNS%
echo DC: %DC%
echo DC DNS name: %DCDNS%
echo DC IP Address: %DCIP%
echo Report directory: %ADir%\%RepDir%
echo.
if not exist %ADir%\Reports md %ADir%\Reports
if not exist %ADir%\%RepDir% md %ADir%\%RepDir%
if not exist %ADir%\%RepDir%\GPMC md %ADir%\%RepDir%\GPMC
set CindyDir=%ADir%\%RepDir%\DCConfig
if not exist %CindyDir% md %CindyDir%

echo Documenting AD data using Jose ...
rem try both language versions, probably only one exists
if exist %ADir%\jose\Standard-Reports.bat call %ADir%\jose\Standard-Reports.bat
if exist %ADir%\jose\Default-Reports.bat call %ADir%\jose\Default-Reports.bat
rem copy José's reports to the report folder
md %ADir%\%RepDir%\Jose
md %ADir%\%RepDir%\Jose\Reports
md %ADir%\%RepDir%\Jose\Reports\GPMC-Reports
md %ADir%\%RepDir%\Jose\Reports\Dateien
move %ADir%\jose\Reports\*.* %ADir%\%RepDir%\Jose\Reports > NUL
move %ADir%\jose\Reports\GPMC-Reports\*.* %ADir%\%RepDir%\Jose\Reports\GPMC-Reports > NUL
copy %ADir%\jose\Reports\Dateien\*.* %ADir%\%RepDir%\Jose\Reports\Dateien > NUL
echo.

echo Documenting AD sites using Borg ...
cscript //nologo %ADir%\Borg\Borg.vbs
rem copy Borg's reports to the report folder
md %ADir%\%RepDir%\Borg
md %ADir%\%RepDir%\Borg\Dateien
md %ADir%\%RepDir%\Borg\Reports
copy %ADir%\Borg\Dateien\*.* %ADir%\%RepDir%\Borg\Dateien > NUL
move %ADir%\Borg\Reports\*.* %ADir%\%RepDir%\Borg\Reports > NUL
echo.

if not exist "%ProgramFiles%\GPMC\Scripts\." (
   echo GPMC VBS scripts not found
   goto NoGPMCVBS
)
echo Documenting Group Policies using GPMC VBS scripts ...
cscript //nologo %ProgramFiles%\GPMC\Scripts\GetReportsForAllGPOs.wsf %ADir%\%RepDir%\GPMC
echo.
goto AfterGPMC

:NoGPMCVBS
echo Trying to report Group Policies using PowerShell ...
start powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -command "& {Get-GPOReport -All -Path %ADir%\%RepDir%\GPMC\AllGPO.html -ReportType Html}"
start powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -file  %ADir%\jose\Get-GPMCReports.ps1

:AfterGPMC
echo Documenting groups using PowerShell ...
start /wait powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -file %Adir%\Get-GroupReport\Get-GroupReport.ps1
rem Get-GroupReport.ps1 writes its output to its own script folder so we move it to the Reports folder
md %ADir%\%RepDir%\GroupReport
move /Y %Adir%\Get-GroupReport\GroupReport*.csv %ADir%\%RepDir%\GroupReport > NUL

echo Documenting circular nested groups using PowerShell ...
start /wait powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -command "& {%Adir%\Get-CircularNestedADGroups\Get-CircularNestedADGroups.ps1 | Sort-Object Name | ConvertTo-Html -PreContent 'Circular Nested Groups: If this report is empty, there are no Circular Nested Groups.' -Title 'Circular Nested Groups' -property DistinguishedName,Name,GroupScope,GroupCategory | Out-File %ADir%\%RepDir%\CircularGroups.html}"

echo Exporting all objects with Exchange Extension attributes ...
rem create a default file that may be overwritten if csvde.exe finds values
echo No Extension Attributes are in use.>%ADir%\%RepDir%\ExtensionAttributes.txt
csvde -f %ADir%\%RepDir%\ExtensionAttributes.txt -r "(|(extensionAttribute1=*)(extensionAttribute2=*)(extensionAttribute3=*)(extensionAttribute4=*)(extensionAttribute5=*)(extensionAttribute6=*)(extensionAttribute7=*)(extensionAttribute8=*)(extensionAttribute9=*)(extensionAttribute10=*)(extensionAttribute11=*)(extensionAttribute12=*)(extensionAttribute13=*)(extensionAttribute14=*)(extensionAttribute15=*))" -l extensionAttribute1,extensionAttribute2,extensionAttribute3,extensionAttribute4,extensionAttribute5,extensionAttribute6,extensionAttribute7,extensionAttribute8,extensionAttribute9,extensionAttribute10,extensionAttribute11,extensionAttribute12,extensionAttribute13,extensionAttribute14,extensionAttribute15

echo Exporting UPN data ...
rem Users with no UPN
csvde -f %ADir%\%RepDir%\No-UPN.txt -u -r "(&(objectClass=user)(objectCategory=person)(!userPrincipalName=*))" -l sAMAccountName
rem Users with a non-standard UPN
csvde -f %ADir%\%RepDir%\Non-Standard-UPN.txt -u -r "(&(objectClass=user)(objectCategory=person)(!userPrincipalName=*%USERDNSDOMAIN%)(userPrincipalName=*))" -l sAMAccountName,userPrincipalname
rem All UPNs and (primary) mail addresses
csvde -f %ADir%\%RepDir%\MailAndUPN.txt -u -r "(&(objectClass=user)(objectCategory=person)(userPrincipalName=*)(mail=*))" -l sAMAccountName,userPrincipalName,mail

echo Analysing DNS using DNSlint ...
cd %ADir%\%RepDir%
start %ADir%\DNSLint\dnslint /ad %DCIP% /s %DCIP% /test_tcp /no_open /y
start %ADir%\DNSLint\dnslint /d %DOMDNS% /s %DCIP% /r DNSLint-Delegation /no_open /y
cd..
echo.

echo Documenting DC status using DCDiag ...
dcdiag /E /s:%DC% >%ADir%\%RepDir%\DCDiag.txt
dcdiag /test:RidManager /v | find /i "available rid pool" >%ADir%\%RepDir%\RIDPool.txt
echo.

echo Documenting stale objects using OldCMP ...
cd %ADir%\%RepDir%
%ADir%\OldCMP\oldcmp -report -llts -file OldComputers-LLTS.htm
%ADir%\OldCMP\oldcmp -report -file OldComputers-PWD.htm
%ADir%\OldCMP\oldcmp -report -llts -af "(operatingSystem=*server*)" -file OldServers-LLTS.htm
%ADir%\OldCMP\oldcmp -report -af "(operatingSystem=*server*)" -file OldServers-PWD.htm
%ADir%\OldCMP\oldcmp -report -users -llts -file OldUsers-LLTS.htm
%ADir%\OldCMP\oldcmp -report -users -file OldUsers-PWD.htm
cd..
echo.

echo Documenting AD objects using ADFind ...
cd %ADir%\%RepDir%
rem Domain SID
for /f %%i in ('%ADir%\Adfind\adfind -list -s Base objectSID') do set DomainSID=%%i
echo %DomainSID% > DomainSID.txt
rem Default objects
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=%DomainSID%-512>" -list name') do set DomainAdmins=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=%DomainSID%-512>" -list distinguishedName') do set DomainAdminsDN=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=%DomainSID%-513>" -list name') do set DomainUsers=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=%DomainSID%-513>" -list distinguishedName') do set DomainUsersDN=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=S-1-5-32-544>" -list name') do set Administrators=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=S-1-5-32-544>" -list distinguishedName') do set AdministratorsDN=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=S-1-5-32-545>" -list name') do set Users=%%i
for /f "Tokens=*" %%i in ('%ADir%\Adfind\adfind -b "<SID=S-1-5-32-545>" -list distinguishedName') do set UsersDN=%%i
echo Domain Administrators group name: %DomainAdmins%>DefaultObjects.txt
echo Domain Administrators group DN: %DomainAdminsDN%>>DefaultObjects.txt
echo Domain Users group name: %DomainUsers%>>DefaultObjects.txt
echo Domain Users group DN: %DomainUsersDN%>>DefaultObjects.txt
echo BUILTIN\Administrators group name: %Administrators%>>DefaultObjects.txt
echo BUILTIN\Administrators group DN: %AdministratorsDN%>>DefaultObjects.txt
echo BUILTIN\Users group name: %Users%>>DefaultObjects.txt
echo BUILTIN\Users group DN: %UsersDN%>>DefaultObjects.txt
%ADir%\Adfind\adfind -b "<SID=%DomainSID%-512>" memberOf > DefaultGroupsMemberOf.txt
%ADir%\Adfind\adfind -b "<SID=%DomainSID%-513>" memberOf >> DefaultGroupsMemberOf.txt
%ADir%\Adfind\adfind -b "<SID=S-1-5-32-544>" memberOf >> DefaultGroupsMemberOf.txt
%ADir%\Adfind\adfind -b "<SID=S-1-5-32-545>" memberOf >> DefaultGroupsMemberOf.txt
rem user machine account quota
%ADir%\Adfind\adfind -list -default -s base MS-DS-MachineAccountQuota > MachineAccountQuota.txt
rem Objects with AdminCount=1
%ADir%\Adfind\adfind -sc admincountdmp memberOf primaryGroupID > Admincount.txt
%ADir%\Adfind\adfind -f "(&(objectClass=group)(admincount=1))" member > Admincount-Groups.txt
rem Non-Standard Primary Groups (not 513)
%ADir%\Adfind\adfind -f "(&(objectClass=user)(objectCategory=person)(!(primaryGroupID=513)))" dn sAMAccountName primaryGroupID memberOf > Users-PrimaryGroupIDNot513.txt
%ADir%\Adfind\adfind -f "(&(objectClass=user)(objectCategory=person)(!(primaryGroupID=513)))" -list primaryGroupID > PrimaryGroupIDsInUse.txt
rem Objects with SID History
%ADir%\Adfind\adfind -f sidhistory=* sidhistory  -recmute -csv -csvdelim ; name sAMAccountName sidhistory objectClass > SIDHistory.txt
echo Number of objects with SID History > NumberObjectsSIDHistory.txt
%ADir%\Adfind\adfind -c -list -f "(sidhistory=*)" >> NumberObjectsSIDHistory.txt
rem number of direct and indirect group memberships
%ADir%\Adfind\adfind -default -f "(&(objectClass=user)(objectCategory=person))" distinguishedName -list | %ADir%\Adfind\adfind -s base memberOf tokenGroups -cv -csv > NumberGroupMembershipsPerUser.txt
rem direct group memberships
%ADir%\Adfind\adfind -default -f "(&(objectClass=user)(objectCategory=person))" distinguishedName -list | %ADir%\Adfind\adfind -s base memberOf > UsersMemberOf.txt
rem indirect group memberships
%ADir%\Adfind\adfind -default -f "(&(objectClass=user)(objectCategory=person))" distinguishedName -list | %ADir%\Adfind\adfind -s base tokengroups -resolvesids > UsersTokenGroups.txt

rem Translate PrimaryGroupIDs to group names in CSV format
setlocal EnableDelayedExpansion
del PrimaryGroupIDsInUse-Unique.txt 2>nul
set row=###
for /F "delims=" %%i in ('type "PrimaryGroupIDsInUse.txt" ^| sort') do (
  if /I not "%%i"=="!row!" (echo.%%i >> "PrimaryGroupIDsInUse-Unique.txt" & set row=%%i)
)
setlocal DisableDelayedExpansion

echo "distinguishedName","name","primaryGroupToken">PrimaryGroupsInUse-Names.txt
for /F " usebackq delims==" %%i in (`@type "PrimaryGroupIDsInUse-Unique.txt"`) do (
  echo %DomainSID%-%%i
  %ADir%\Adfind\adfind -csv -nocsvheader -sc adsid:%DomainSID%-%%i name primaryGroupToken>>PrimaryGroupsInUse-Names.txt
)

ping -n 1 127.0.0.1>nul

rem Domain Controllers
%ADir%\Adfind\adfind -sc !gclist > DomainControllers-NotGC.txt
%ADir%\Adfind\adfind -sc dcdmp > DomainControllers.txt

rem Relevant object count
del ObjectCount.txt 2>nul
del ObjCntTmp.txt 2>nul
echo user: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:user >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo contact: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:contact >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo computer: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:computer >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo group: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:group >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo ou: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:ou >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo site: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:site >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo subnet: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:subnet >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo gpo: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:gpo >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo foreign security principals: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:fsp >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
echo mailbox: >>ObjCntTmp.txt && %ADir%\Adfind\adfind -list -sc adobjcnt:mailbox >> ObjCntTmp.txt && echo -->>ObjCntTmp.txt
rem Remove blank lines from output file
type nul>ObjectCount.txt
for /f "delims=" %%i in (ObjCntTmp.txt) do echo %%i>> ObjectCount.txt
del ObjCntTmp.txt 2>nul

rem Dump critical objects
%ADir%\Adfind\adfind -list -sc users_disabled distinguishedName>users_disabled.txt
%ADir%\Adfind\adfind -list -sc users_noexpire distinguishedName>users_noexpire.txt
%ADir%\Adfind\adfind -list -sc users_accexpired distinguishedName>users_accexpired.txt
%ADir%\Adfind\adfind -list -sc users_pwdnotreqd distinguishedName>users_pwdnotreqd.txt
%ADir%\Adfind\adfind -list -sc computers_disabled distinguishedName>computers_disabled.txt
%ADir%\Adfind\adfind -list -sc computers_pwdnotreqd distinguishedName>computers_pwdnotreqd.txt
%ADir%\Adfind\adfind -list -sc computers_active distinguishedName>computers_active.txt
%ADir%\Adfind\adfind -list -sc computers_inactive distinguishedName>computers_inactive.txt

echo Collecting local OS configuration data for all DCs ...
echo Computer Name;Domain;VM;OS;Service Pack;NIC Adapter Name;MAC Address;DHCP Enabled;IP Addresses;Subnets;Gateway;DNS Servers;RAM;DC;Collect Date>"%CindyDir%\DCData.txt"
for /F "delims=" %%i in ('%ADir%\Adfind\adfind -sc dclist') do (
  echo ... %%i ...
  md %CindyDir%\%%i
  CSCRIPT //nologo //u "%ADir%\Cindy\cindy.vbs" %%i > "%CindyDir%\%%i\%%i-Quick.txt"
  CSCRIPT //nologo //u "%ADir%\Cindy\cindy.vbs" %%i /mode:basic > "%CindyDir%\%%i\%%i-Basic.txt"
  CSCRIPT //nologo "%ADir%\Costa\costa.vbs" %%i /header:no>> "%CindyDir%\DCData.txt"
)

echo Analyzing AD Schema using PowerShell ...
cd %ADir%\%RepDir%
start powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -file %Adir%\Schema\Count-SchemaChanges.ps1
start powershell -noninteractive -noprofile -ExecutionPolicy unrestricted -file %Adir%\Schema\Compare-SchemaObjectsToTemplate.ps1 -OutputFilePath %ADir%\%RepDir%\LocalSchemaObjects.txt -TemplatePath %Adir%\Schema\schemaobjects.txt -DifferenceFilePath %ADir%\%RepDir%\UnknownSchemaObjects.txt

cd %ADir%
echo Collecting report ...
cscript //nologo %ADir%\Reports\Collect-Report.vbs "%Adir%\%RepDir%" "%DOM%" "%MyDate%" "%ADir%" "%ADir%\%RepDir%"
echo.
echo Finished at: %time%
echo Find the reports at %ADir%\%RepDir%. 
echo Jose and Borg reports are stored under their respective folders.
echo View the full report overview at %Adir%\%RepDir%\_Report-Overview.html
echo.
